@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef ASYNCH
-#define ASYNCH
-/* Copyright (C) 1995, 96, 97, 98, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_ASYNC_H
+#define SCM_ASYNC_H
+/* Copyright (C) 1995, 96, 97, 98, 2000, 2001 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,11 +47,10 @@
 
 #include "libguile/__scm.h"
 
-
 
 
 extern unsigned int scm_mask_ints;
-extern SCM scm_gc_async;
+
 
 
 extern int scm_asyncs_pending (void);
@@ -69,7 +68,7 @@ extern SCM scm_unmask_signals (void);
 extern SCM scm_mask_signals (void);
 extern void scm_init_async (void);
 
-#endif  /* ASYNCH */
+#endif  /* SCM_ASYNC_H */
 
 /*
   Local Variables:
