@@ -3,7 +3,7 @@
 #ifndef COOP_THREADSH
 #define COOP_THREADSH
 
-/*	Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,8 +55,8 @@
 
 #include <time.h>
 
-#include "coop-defs.h"
-#include "../qt/qt.h"
+#include "libguile/coop-defs.h"
+#include "qt/qt.h"
 
 /* This code is based on a sample thread libraru by David Keppel.
    Portions of this file fall under the following copyright: */
@@ -122,3 +122,9 @@ extern coop_q_t coop_global_allq;	/* A queue of all threads. */
 extern coop_t *coop_global_curr;       	/* Currently-executing thread. */
 
 #endif /* COOP_THREADSH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
