@@ -2,7 +2,7 @@
 
 #ifndef EQH
 #define EQH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996, 2000 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -40,16 +41,21 @@
  *
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  
- */
+ * If you do not wish that, delete this exception notice.  */
 
 #include "libguile/__scm.h"
 
 
 
-extern SCM scm_eq_p SCM_P ((SCM x, SCM y));
-extern SCM scm_eqv_p SCM_P ((SCM x, SCM y));
-extern SCM scm_equal_p SCM_P ((SCM x, SCM y));
-extern void scm_init_eq SCM_P ((void));
+extern SCM scm_eq_p (SCM x, SCM y);
+extern SCM scm_eqv_p (SCM x, SCM y);
+extern SCM scm_equal_p (SCM x, SCM y);
+extern void scm_init_eq (void);
 
 #endif  /* EQH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
