@@ -1,4 +1,4 @@
-/*	Copyright (C) 1997, 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+/*	Copyright (C) 1997, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -184,7 +184,7 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
 {
   SCM flag;
   regex_t *rx;
-  int status, cflags, argnum;
+  int status, cflags;
 
   SCM_VALIDATE_STRING (1, pat);
   SCM_VALIDATE_REST_ARGUMENT (flags);
@@ -194,17 +194,13 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
      turn off REG_EXTENDED flag (on by default). */
   cflags = REG_EXTENDED;
   flag = flags;
-  argnum = 2;
   while (!SCM_NULLP (flag))
     {
-      int f;
-      SCM_VALIDATE_INT_COPY (argnum, SCM_CAR (flag), f);
-      if (f == REG_BASIC)
+      if (SCM_INUM (SCM_CAR (flag)) == REG_BASIC)
 	cflags &= ~REG_EXTENDED;
       else
-	cflags |= f;
+	cflags |= SCM_INUM (SCM_CAR (flag));
       flag = SCM_CDR (flag);
-      argnum++;
     }
 
   rx = SCM_MUST_MALLOC_TYPE (regex_t);
@@ -214,12 +210,9 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
 		    cflags & ~REG_NOSUB);
   if (status != 0)
     {
-      char *errmsg = scm_regexp_error_msg (status, rx);
-      free (rx);
-      scm_done_free (sizeof (regex_t));
       scm_error (scm_regexp_error_key,
 		 FUNC_NAME,
-                 errmsg,
+		 scm_regexp_error_msg (status, rx),
 		 SCM_BOOL_F,
 		 SCM_BOOL_F);
       /* never returns */
