@@ -1,19 +1,19 @@
 /* classes: h_files */
 
-#ifndef TIMEH
-#define TIMEH
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
- * 
+#ifndef STIMEH
+#define STIMEH
+/*	Copyright (C) 1995,1996,1997,1998, 2000 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
@@ -47,16 +47,24 @@
 #include "libguile/__scm.h"
 
 
-extern SCM scm_get_internal_real_time SCM_P ((void));
-extern SCM scm_get_internal_run_time SCM_P ((void));
-extern SCM scm_current_time SCM_P ((void));
+long scm_c_get_internal_run_time (void);
+extern SCM scm_get_internal_real_time (void);
+extern SCM scm_get_internal_run_time (void);
+extern SCM scm_current_time (void);
 extern SCM scm_gettimeofday (void);
 extern SCM scm_localtime (SCM time, SCM zone);
 extern SCM scm_gmtime (SCM time);
 extern SCM scm_mktime (SCM sbd_time, SCM zone);
 extern SCM scm_tzset (void);
-extern SCM scm_strftime SCM_P ((SCM format, SCM stime));
-extern SCM scm_strptime SCM_P ((SCM format, SCM string));
-extern void scm_init_stime SCM_P ((void));
+extern SCM scm_times (void);
+extern SCM scm_strftime (SCM format, SCM stime);
+extern SCM scm_strptime (SCM format, SCM string);
+extern void scm_init_stime (void);
 
-#endif  /* TIMEH */
+#endif  /* STIMEH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
