@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef UNIFH
-#define UNIFH
-/*	Copyright (C) 1995,1996,1997,1999, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_UNIFORM_VECTORS_H
+#define SCM_UNIFORM_VECTORS_H
+/* Copyright (C) 1995,1996,1997,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,50 +50,72 @@
 
 
 /*
-  an array SCM is a non-immediate pointing to a  heap cell with:
+  an array SCM is a non-immediate pointing to a  heap cell where:
 
-   CAR: bits 0-14 hold the dimension (0 -- 32767)
-        bit  15 is the SCM_ARRAY_CONTIGUOUS flag
-        bits 16-31 hold the smob type id: scm_tc16_array
-   CDR: pointer to a malloced block containing an scm_array structure
-        followed by an scm_array_dim structure for each dimension.
+   CAR: bits 0-15 hold the smob type id: scm_tc16_array
+        bit  16 is the SCM_ARRAY_FLAG_CONTIGUOUS flag
+	bits 17-31 hold the dimension (0 -- 32767)
+   CDR: pointer to a malloced block containing an scm_t_array structure
+        followed by an scm_t_array_dim structure for each dimension.
 */
 
-typedef struct scm_array
+typedef struct scm_t_array
 {
   SCM v;  /* the contents of the array, e.g., a vector or uniform vector.  */
-  scm_sizet base;
-} scm_array;
+  unsigned long base;
+} scm_t_array;
 
-typedef struct scm_array_dim
+typedef struct scm_t_array_dim
 {
   long lbnd;
   long ubnd;
   long inc;
-} scm_array_dim;
+} scm_t_array_dim;
 
+#if (SCM_DEBUG_DEPRECATED == 0)
+# define scm_array scm_t_array
+# define scm_array_dim scm_t_array_dim
+#endif
 
-extern long scm_tc16_array;
-#define SCM_ARRAYP(a) 		(SCM_NIMP(a) && (scm_tc16_array == SCM_TYP16(a)))
-#define SCM_ARRAY_NDIM(x) 	((scm_sizet) (SCM_CELL_WORD_0 (x) >> 17))
-#define SCM_ARRAY_CONTIGUOUS 	0x10000
-#define SCM_ARRAY_CONTP(x) 	(SCM_ARRAY_CONTIGUOUS & (SCM_CELL_WORD_0 (x)))
+extern scm_t_bits scm_tc16_array;
 
-#define SCM_ARRAY_V(a) 	  (((scm_array *) SCM_CELL_WORD_1 (a))->v)
-#define SCM_ARRAY_BASE(a) (((scm_array *) SCM_CELL_WORD_1 (a))->base)
-#define SCM_ARRAY_DIMS(a) ((scm_array_dim *)(SCM_CHARS(a)+sizeof(scm_array))) 
+#define SCM_ARRAY_FLAG_CONTIGUOUS (1 << 16)
 
-/* apparently it's possible to have more than SCM_LENGTH_MAX elements
-   in an array: if the length is SCM_LENGTH_MAX then the SCM_VELTS
-   block begins with the true length (a long int).  I wonder if it
-   works.  */
+#if (SCM_DEBUG_DEPRECATED == 0)
+#define SCM_ARRAY_CONTIGUOUS SCM_ARRAY_FLAG_CONTIGUOUS
+#endif
 
-#define SCM_HUGE_LENGTH(x)\
-  (SCM_LENGTH_MAX==SCM_LENGTH(x) ? *((long *)SCM_VELTS(x)) : SCM_LENGTH(x))
+#define SCM_ARRAYP(a) 	    SCM_TYP16_PREDICATE (scm_tc16_array, a)
+#define SCM_ARRAY_NDIM(x)   ((size_t) (SCM_CELL_WORD_0 (x) >> 17))
+#define SCM_ARRAY_CONTP(x)  (SCM_CELL_WORD_0 (x) & SCM_ARRAY_FLAG_CONTIGUOUS)
+#define SCM_SET_ARRAY_CONTIGUOUS_FLAG(x) \
+  (SCM_SET_CELL_WORD_0 ((x), SCM_CELL_WORD_0 (x) | SCM_ARRAY_FLAG_CONTIGUOUS))
+#define SCM_CLR_ARRAY_CONTIGUOUS_FLAG(x) \
+  (SCM_SET_CELL_WORD_0 ((x), SCM_CELL_WORD_0 (x) & ~SCM_ARRAY_FLAG_CONTIGUOUS))
+
+#define SCM_ARRAY_MEM(a)  ((scm_t_array *) SCM_CELL_WORD_1 (a))
+#define SCM_ARRAY_V(a) 	  (SCM_ARRAY_MEM (a)->v)
+#define SCM_ARRAY_BASE(a) (SCM_ARRAY_MEM (a)->base)
+#define SCM_ARRAY_DIMS(a) ((scm_t_array_dim *)((char *) SCM_ARRAY_MEM (a) + sizeof (scm_t_array))) 
+
+#define SCM_I_MAX_LENGTH  ((unsigned long) (-1L) >> 8)
+
+#define SCM_UVECTOR_BASE(x) ((void *) (SCM_CELL_WORD_1 (x)))
+#define SCM_SET_UVECTOR_BASE(v, b) (SCM_SET_CELL_WORD_1 ((v), (b)))
+#define SCM_UVECTOR_MAX_LENGTH SCM_I_MAX_LENGTH
+#define SCM_UVECTOR_LENGTH(x) (((unsigned long) SCM_CELL_WORD_0 (x)) >> 8)
+#define SCM_SET_UVECTOR_LENGTH(v, l, t) (SCM_SET_CELL_WORD_0 ((v), ((l) << 8) + (t)))
+
+#define SCM_BITVECTOR_P(x) (!SCM_IMP (x) && (SCM_TYP7 (x) == scm_tc7_bvect))
+#define SCM_BITVECTOR_BASE(x) ((unsigned long *) (SCM_CELL_WORD_1 (x)))
+#define SCM_SET_BITVECTOR_BASE(v, b) (SCM_SET_CELL_WORD_1 ((v), (b)))
+#define SCM_BITVECTOR_MAX_LENGTH SCM_I_MAX_LENGTH
+#define SCM_BITVECTOR_LENGTH(x) (((unsigned long) SCM_CELL_WORD_0 (x)) >> 8)
+#define SCM_SET_BITVECTOR_LENGTH(v, l) (SCM_SET_CELL_WORD_0 ((v), ((l) << 8) + scm_tc7_bvect))
 
 
 
-extern scm_sizet scm_uniform_element_size (SCM obj);
+extern size_t scm_uniform_element_size (SCM obj);
 extern SCM scm_make_uve (long k, SCM prot);
 extern SCM scm_uniform_vector_length (SCM v);
 extern SCM scm_array_p (SCM v, SCM prot);
@@ -112,7 +134,7 @@ extern SCM scm_transpose_array (SCM ra, SCM args);
 extern SCM scm_enclose_array (SCM ra, SCM axes);
 extern SCM scm_array_in_bounds_p (SCM v, SCM args);
 extern SCM scm_uniform_vector_ref (SCM v, SCM args);
-extern SCM scm_cvref (SCM v, scm_sizet pos, SCM last);
+extern SCM scm_cvref (SCM v, unsigned long pos, SCM last);
 extern SCM scm_array_set_x (SCM v, SCM obj, SCM args);
 extern SCM scm_array_contents (SCM ra, SCM strict);
 extern SCM scm_ra2contig (SCM ra, int copy);
@@ -130,7 +152,20 @@ extern int scm_raprin1 (SCM exp, SCM port, scm_print_state *pstate);
 extern SCM scm_array_prototype (SCM ra);
 extern void scm_init_unif (void);
 
-#endif  /* UNIFH */
+
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+
+/* apparently it's possible to have more than SCM_LENGTH_MAX elements
+   in an array: if the length is SCM_LENGTH_MAX then the SCM_VELTS
+   block begins with the true length (a long int).  I wonder if it
+   works.  */
+#define SCM_HUGE_LENGTH(x)\
+  (SCM_LENGTH_MAX==SCM_LENGTH(x) ? *((long *)SCM_VELTS(x)) : SCM_LENGTH(x))
+
+#endif  /* SCM_DEBUG_DEPRECATED == 0 */
+
+#endif  /* SCM_UNIFORM_VECTORS_H */
 
 /*
   Local Variables:
